<?php

namespace ProcessWire;

class TurboPW {
    
    private $lang;
    public $wire;
    public $max_menu_level=2;

    public function __construct($wire, $lang = null) {
        $this->wire = $wire;
        if ($lang == null)
            $this->lang = $this->wire->user->language;
        else
            $this->lang = $lang;
    }

    public function setLang($newval) {
        $this->lang = $newval;
    }

    private function getLang() {
        return $this->lang;
    }

    public function findObject($selector, $fields, $fields_pages) {
        return $this->findArray($selector, $fields, $fields_pages, true);
    }

    public function findArray($selector, $fields, $fields_pages = [], $type = \PDO::FETCH_ASSOC) {
        $savedlang = $this->wire->user->language;
        // build sql string
        $sql = "SELECT\n  p.";

        // add fields of pages table
        $fields_pages[] = 'id'; // make sure we return the page id
        $fields_pages = array_unique($fields_pages);
        $sql .= implode(",\n  p.", $fields_pages);

        foreach ($fields as $f) {
            $field_extra = explode(':', $f);
            $f = $field_extra[0];
            $field_extra = array_slice($field_extra, 1);
            $field = $this->wire->fields->get($f);

            if (!$field)
                continue;
            $fieldtype = $field->type;
            $f = strtolower($f);
            // fielddata is always stored in the "data" column of the field's table
            // multilang fields have several data columns identified by the language id
            // we use a variable to query the current user's language, eg data1234
            $data = "data";
            //echo '<b> '.$field->name.' '.$fieldtype.' ('.($fieldtype instanceof FieldtypeText).') </b>';
            switch (true) {
                // if it is a multilang field we append the language id to query the correct column
                case $fieldtype instanceof FieldtypeTextLanguage:
                case $fieldtype instanceof FieldtypeTextareaLanguage:
                    if ($this->lang->name != 'default')
                        $data .= $this->lang->id;
                // no break here intended!
                // build sql query
                case $fieldtype instanceof FieldtypeText:
                case $fieldtype instanceof FieldtypeInteger:
                case $fieldtype instanceof FieldtypeFloat:
                case $fieldtype instanceof FieldtypeCheckbox:
                    //case $fieldtype instanceof FieldtypeTextarea: //is a fieldtypetext
                    //case $fieldtype instanceof FieldtypeDatetime://is a fieldtypetext
                    $sql .= ",\n  (SELECT $data FROM field_$f WHERE pages_id = p.id) AS $f";
                    break;
                case $fieldtype instanceof FieldtypePage:
                case $fieldtype instanceof FieldtypeRepeater:
                    $sql .= ",\n  (SELECT GROUP_CONCAT($data SEPARATOR ',') FROM field_$f WHERE pages_id = p.id) AS $f";
                    break;
                case $fieldtype instanceof FieldtypeImage:
                case $fieldtype instanceof FieldtypeFile:
                    $sql .= ",\n  (SELECT GROUP_CONCAT($data SEPARATOR 0x1D) FROM field_$f WHERE pages_id = p.id) AS $f";
                    foreach ($field_extra as $exfld)
                        $sql .= ",\n  (SELECT GROUP_CONCAT($exfld SEPARATOR 0x1D) FROM field_$f WHERE pages_id = p.id) AS " . $f . "_" . $exfld;
                    break;
                case $fieldtype instanceof FieldtypeOptions:
                    $sql .= ",\n  (SELECT $data FROM field_$f WHERE pages_id = p.id) AS $f";
                    if (in_array('title', $field_extra))
                        $sql .= ",\n  (SELECT title FROM fieldtype_options WHERE fields_id=(SELECT id FROM fields WHERE name = '$f') AND option_id=$f) AS " . $f . "_title";
                    break;
                default:
                    $sql .= ",\n  '$fieldtype not supported' AS $f";
            }
        }
        $pages = $this->wire->pages->findIDs($selector);
        if (count($pages) == 0) {
            return [];
        } else {
            $pagesset = implode(",", $pages);
            $sql .= "\nFROM\n  pages AS p";
            $sql .= "\nWHERE\n  p.id IN (" . $pagesset . ")";
            $sql .= "\nORDER BY FIELD(p.id," . $pagesset . ")";
            //echo '<pre>' . $sql . '</pre>';
            $results = $this->wire->database->query($sql);
            return $results->fetchAll($type);
        }
    }

    public function getFiles($images, $images_desc = []) {
        $images = explode(chr(29), $images);

        $langid = $this->lang->id;
        if ($langid == $this->wire->languages->getDefault()->id)
            $langid = 0;

        $result = [];

        if (count($images_desc) > 0) {
            $images_desc = explode(chr(29), $images_desc);
            $imgs = array_combine($images, $images_desc);
            foreach ($imgs as $url => $descr) {
                $ajson = json_decode($descr, true);
                if (isset($ajson[$langid]))
                    $desc = $ajson[$langid];
                else
                    $desc = "";
                $result[] = ['url' => $url, 'desc' => $desc];
            }
        } else
            $result = $images;
        return $result;
    }

    public function buildTree($flat, $pidKey, $idKey = null) {
        $grouped = array();
        foreach ($flat as $sub) {
            $grouped[$sub[$pidKey]][] = $sub;
        }
        $fnBuilder = function($siblings) use (&$fnBuilder, $grouped, $idKey) {
            foreach ($siblings as $k => $sibling) {
                $id = $sibling[$idKey];
                if (isset($grouped[$id])) {
                    $sibling['children'] = $fnBuilder($grouped[$id]);
                }
                $siblings[$k] = $sibling;
            }
            return $siblings;
        };
        $tree = $fnBuilder($grouped[$flat[0][$pidKey]]); //!!
        return $tree;
    }

    private function echoBS4children($carr, $path, $level, $actives) {
        foreach ($carr as $p) {
            $href = $path . $p['name'] . '/';
            $activestr = (in_array($p["id"], $actives) ? ' active' : '');
            if (isset($p['children']) && $level < $this->max_menu_level) {
                echo '<li><a class="dropdown-item dropdown-toggle dropdown-sub' . $activestr . '" href="' . $href . '">' . $p['title'] . '</a>';
                echo "\n".'<ul class="dropdown-menu level_' . $level . '">'."\n";
                $this->echoBS4children($p['children'], $href, $level + 1, $actives);
                echo '</ul></li>'."\n";
            } else {
                echo '<li><a class="dropdown-item' . $activestr . '" href="' . $href . '">' . $p['title'] . '</a></li>'."\n";
            }
        }
    }

    public function echoBS4Nav($selector, $starturl) {
        $arr = $this->findArray($selector, array('title'), array('id', 'parent_id', 'name'));
        $tree = $this->buildTree($arr, 'parent_id', 'id');

        $thispage = $this->wire->page;
        //$thispage=$this->wire->pages->get(2691);
        $parents = $thispage->parentsUntil(1);

        $actives = [];
        foreach ($parents as $p)
            $actives[] = $p->id;
        $actives[] = $thispage->id;
        echo '<ul class="navbar-nav mr-auto">';
        foreach ($tree as $p) {
            if ($p["id"] == 1)
                $p["name"] = '';
            $href = $starturl . $p['name'] . '/';
            $activestr = (in_array($p["id"], $actives) ? ' active' : '');
            if (isset($p['children'])) {
                echo '<li class="nav-item dropdown' . $activestr . '">';
                echo '<a class="nav-link dropdown-toggle" href="' . $href . '" id="navbarDropdownMenuLink' . $p['id'] . '" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">';
                echo $p['title'] . '</a>';
                echo "\n".'<ul class="dropdown-menu level_0" aria-labelledby="navbarDropdownMenuLink' . $p['id'] . '">';
                $this->echoBS4children($p['children'], $href, 1, $actives);
                echo '</ul></li>'."\n";
            } else {
                echo '<li class="nav-item' . $activestr . '">';
                echo '<a class="nav-link" href="' . $href . '">' . $p['title'] . '</a>';
                echo '</li>'."\n";
            }
        }
        echo '</ul>';
    }

}
